package com.ym.mec.auth.service.impl;

import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.api.enums.YesOrNoEnum;
import com.ym.mec.auth.dal.dao.SysUserDao;
import com.ym.mec.auth.service.SysRoleMenuService;
import com.ym.mec.auth.service.SysUserRoleService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.web.TeacherFeignService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class SysUserServiceImpl extends BaseServiceImpl<Integer, SysUser> implements SysUserService {

	@Autowired
	private SysUserDao sysUserDao;

	@Autowired
	private SysUserRoleService sysUserRoleService;

	@Autowired
	private TeacherFeignService teacherFeignService;

	@Autowired
	private SysRoleMenuService sysRoleMenuService;

	@Autowired
	private ImFeignService imFeignService;

	@Value("${message.autoRegister}")
	private boolean autoRegister;

	@Override
	public BaseDAO<Integer, SysUser> getDAO() {
		return sysUserDao;
	}

	@Override
	public SysUser queryByUsername(String username) {
		return sysUserDao.queryByUsername(username);
	}

	@Override
	public SysUser queryByPhone(String phone) {
		return sysUserDao.queryByPhone(phone);
	}

	@Override
	public SysUserInfo queryUserInfoByUsername(String username) {
		SysUser sysUser = queryByUsername(username);
		return getSysUserInfo(sysUser);
	}

	@Override
	public SysUserInfo queryUserInfoByPhone(String phone) {
		SysUser sysUser = queryByPhone(phone);
		return getSysUserInfo(sysUser);
	}

	private SysUserInfo getSysUserInfo(SysUser sysUser) {
		if (sysUser == null) {
			return null;
		}
		SysUserInfo userInfo = new SysUserInfo();
		userInfo.setSysUser(sysUser);

		List<String> roleCodeList = sysUserRoleService.queryRoleCodeListByUserId(sysUser.getId());
		userInfo.setRoles(roleCodeList.toArray(new String[roleCodeList.size()]));

		List<Integer> roleIdList = sysUserRoleService.queryRoleIdListByUserId(sysUser.getId());
		List<String> permissionList = sysRoleMenuService.queryPermissionsByRoleIdList(roleIdList);

		userInfo.setPermissions(permissionList.toArray(new String[permissionList.size()]));

		return userInfo;
	}

	@Override
	public boolean deleteById(Integer id) {
		SysUser user = get(id);

		if (user == null) {
			throw new BizException("账户不存在");
		}

		user.setUpdateTime(new Date());
		user.setDelFlag(YesOrNoEnum.YES);

		update(user);

		return true;
	}

	@Override
	public void updatePassword(String mobile, String password) {
		sysUserDao.updatePassword(mobile,password);
	}

	@Override
	public Integer add(SysUser sysUser) {
		sysUserDao.insert(sysUser);
		switch (sysUser.getUserType()){
			case TEACHER:

		}
		ImResult imResult = imFeignService.register(new ImUserModel(sysUser.getId().toString(), sysUser.getUsername(), sysUser.getAvatar()));
		sysUser.setImToken(imResult.getToken());
		sysUserDao.update(sysUser);
		return sysUser.getId();
	}

	@Override
	public ImUserModel getBasic(Integer userId) {
		return sysUserDao.getBasic(userId);
	}

	@Override
	public void updateBaseInfo(SysUser sysUser) {
		sysUserDao.update(sysUser);
		imFeignService.update(new ImUserModel(sysUser.getId().toString(),sysUser.getUsername(),null));
	}

	@Override
	public SysUser queryUserInfo(Integer userId) {
		SysUser sysUser = sysUserDao.get(userId);
		sysUser.setOrganName(sysUserDao.getOrganName(sysUser.getOrganId()));
		return sysUser;
	}

	@Override
	public SysUserInfo initUser(String phone, String clientId) {
		if(autoRegister){
			if(StringUtils.equalsIgnoreCase(clientId,"teacher")){
				//注册用户
				SysUser sysUser = registerUser(phone,SysUserType.TEACHER);
				return queryUserInfoByPhone(phone);
			}else if(StringUtils.equalsIgnoreCase(clientId,"student")){
				SysUser sysUser = registerUser(phone,SysUserType.STUDENT);
				//注册现金账户
				teacherFeignService.saveUserCashAccount(sysUser.getId());
				return queryUserInfoByPhone(phone);
			}
			throw new UsernameNotFoundException("404.9");
		}else {
			throw new UsernameNotFoundException("404.9");
		}
	}

	private SysUser registerUser(String phone, SysUserType sysUserType){
		SysUser sysUser = new SysUser();
		sysUser.setPhone(phone);
		sysUser.setUsername(phone);
		sysUser.setOrganId(1);
		sysUser.setGender(1);
		sysUser.setIsSuperAdmin(false);
		//注册用户
		sysUser.setUserType(sysUserType);
		sysUserDao.insert(sysUser);
		ImResult register = imFeignService.register(new ImUserModel(sysUser.getId().toString(), phone, null));
		sysUser.setImToken(register.getToken());
		sysUserDao.update(sysUser);
		return sysUser;
	}

}